/*
    Aurorae Designer: an application to design Aurorae themes.
    Copyright (C) 2010  Martin Gräßlin <kde@martin-graesslin.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/
#include "auroraedesigner.h"
#include "auroraedesignerview.h"
#include "compliancedelegate.h"
#include "compliancemodel.h"
#include "configmodel.h"
#include "previewdelegate.h"
#include "previewmodel.h"
#include "settings.h"

#include <auroraetheme.h>

#include <QtCore/QPointer>
#include <QtGui/QDockWidget>
#include <QtGui/QListView>
#include <QtGui/QTreeView>

#include <kconfigdialog.h>
#include <kstatusbar.h>

#include <KDE/KAction>
#include <KDE/KActionCollection>
#include <KDE/KLocale>
#include <KDE/KNS3/DownloadDialog>
#include <KDE/KStandardAction>
#include <KDE/KTemporaryFile>

AuroraeDesigner::AuroraeDesigner()
    : KXmlGuiWindow()
    , m_printer(0)
{
    m_theme = new Aurorae::AuroraeTheme(this);
    m_view = new AuroraeDesignerView(this, m_theme);
    connect(m_view, SIGNAL(enableRemoveTab(bool)), SLOT(enableRemoveTab(bool)));
    m_tempFile.open();
    m_config = KSharedConfig::openConfig(m_tempFile.fileName(), KConfig::SimpleConfig);
    m_tempDesktopFile.open();
    m_desktopFile = KSharedConfig::openConfig(m_tempDesktopFile.fileName(), KConfig::SimpleConfig);
    // accept dnd
    setAcceptDrops(true);

    // tell the KXmlGuiWindow that this is indeed the main widget
    setCentralWidget(m_view);
    QDockWidget *dock = new QDockWidget(i18n("Decoration Options"), this);
    m_model = new ConfigModel(this);
    connect(m_model, SIGNAL(dataChanged(QModelIndex,QModelIndex)), SLOT(configChanged()));
    QTreeView *view = new QTreeView(this);
    view->setModel(m_model);
    dock->setWidget(view);
    addDockWidget(Qt::RightDockWidgetArea, dock);

    // then, setup our actions
    setupActions();

    // add a status bar
    statusBar()->show();

    // a call to KXmlGuiWindow::setupGUI() populates the GUI
    // with actions, using KXMLGUI.
    // It also applies the saved mainwindow settings, if any, and ask the
    // mainwindow to automatically save settings if changed: window size,
    // toolbar position, icon size, etc.
    setupGUI();
}

AuroraeDesigner::~AuroraeDesigner()
{
}

void AuroraeDesigner::setupActions()
{
//     KStandardAction::openNew(this, SLOT(fileNew()), actionCollection());
    KStandardAction::save(this, SLOT(save()), actionCollection());
    KStandardAction::open(this, SLOT(fileOpen()), actionCollection());
    KStandardAction::quit(qApp, SLOT(closeAllWindows()), actionCollection());

    KStandardAction::preferences(this, SLOT(optionsPreferences()), actionCollection());

    // custom menu and menu item - the slot is in the class AuroraeDesignerView
    KAction *showGridAction = new KAction(KIcon("view-grid"), i18n("Grid"), this);
    showGridAction->setCheckable(true);
    showGridAction->setShortcut(KShortcut("Ctrl+G"));
    actionCollection()->addAction(QLatin1String("show_grid_action"), showGridAction );
    connect(showGridAction, SIGNAL(triggered(bool)), m_view, SLOT(slotShowGrid(bool)));

    KAction *toggleActiveAction = new KAction(KIcon("user-online"), i18n("Active"), this);
    toggleActiveAction->setCheckable(true);
    toggleActiveAction->setChecked(true);
    toggleActiveAction->setShortcut(KShortcut("Ctrl+A"));
    toggleActiveAction->setEnabled(false);
    actionCollection()->addAction(QLatin1String("toggle_active_action"), toggleActiveAction);
    connect(toggleActiveAction, SIGNAL(triggered(bool)), m_view, SLOT(slotActiveChanged(bool)));

    KAction *toggleMaximizeAction = new KAction(KIcon("transform-scale"), i18n("Maximize"), this);
    toggleMaximizeAction->setCheckable(true);
    toggleMaximizeAction->setShortcut(KShortcut("Ctrl+M"));
    toggleMaximizeAction->setEnabled(false);
    actionCollection()->addAction(QLatin1String("toggle_maximize_action"), toggleMaximizeAction);
    connect(toggleMaximizeAction, SIGNAL(triggered(bool)), m_view, SLOT(slotMaximizeChanged(bool)));
    connect(m_view, SIGNAL(maximized(bool)), toggleMaximizeAction, SLOT(setChecked(bool)));

    KAction *toggleCompositingAction = new KAction(KIcon("fill-color"), i18n("Translucency"), this);
    toggleCompositingAction->setCheckable(true);
    toggleCompositingAction->setChecked(true);
    toggleCompositingAction->setShortcut(KShortcut("Ctrl+T"));
    toggleCompositingAction->setEnabled(false);
    actionCollection()->addAction(QLatin1String("toggle_compositing_action"), toggleCompositingAction);
    connect(toggleCompositingAction, SIGNAL(triggered(bool)), SLOT(compositingChanged(bool)));

    KAction *addTabAction = new KAction(KIcon("tab-new"), i18n("Add Window Tab"), this);
    addTabAction->setEnabled(false);
    actionCollection()->addAction(QLatin1String("add_tab_action"), addTabAction);
    connect(addTabAction, SIGNAL(triggered(bool)), m_view, SLOT(addTab()));

    KAction *removeTabAction = new KAction(KIcon("tab-close"), i18n("Remove Window Tab"), this);
    removeTabAction->setEnabled(false);
    actionCollection()->addAction(QLatin1String("remove_tab_action"), removeTabAction);
    connect(removeTabAction, SIGNAL(triggered(bool)), m_view, SLOT(removeTab()));

    KAction *showTooltipsAction = new KAction(i18n("Show Tooltips"), this);
    showTooltipsAction->setCheckable(true);
    showTooltipsAction->setChecked(true);
    showTooltipsAction->setEnabled(false);
    actionCollection()->addAction(QLatin1String("show_tooltips_action"), showTooltipsAction);
    connect(showTooltipsAction, SIGNAL(triggered(bool)), m_theme, SLOT(setShowTooltips(bool)));

    KAction *ghnsAction = new KAction(KIcon("get-hot-new-stuff"), i18n("Get New Themes..."), this);
    actionCollection()->addAction(QLatin1String("ghns_action"), ghnsAction);
    connect(ghnsAction, SIGNAL(triggered(bool)), SLOT(ghns()));

    KAction *complianceAction = new KAction(i18n("Check Compliance..."), this);
    complianceAction->setEnabled(false);
    actionCollection()->addAction(QLatin1String("compliance_action"), complianceAction);
    connect(complianceAction, SIGNAL(triggered(bool)), SLOT(complianceDialog()));

    KAction *reloadAction = new KAction(KIcon("view-refresh"), i18n("Reload"), this);
    reloadAction->setShortcut(KShortcut("Ctrl+F5"));
    reloadAction->setEnabled(false);
    actionCollection()->addAction(QLatin1String("reload_action"), reloadAction);
    connect(reloadAction, SIGNAL(triggered(Qt::MouseButtons,Qt::KeyboardModifiers)), SLOT(reload()));

    QActionGroup *borderSizes = new QActionGroup(this);
    borderSizes->addAction(addBorderAction(i18n("Tiny"), "border_tiny_action", KDecorationDefines::BorderTiny));
    borderSizes->addAction(addBorderAction(i18n("Normal"), "border_normal_action", KDecorationDefines::BorderNormal));
    borderSizes->addAction(addBorderAction(i18n("Large"), "border_large_action", KDecorationDefines::BorderLarge));
    borderSizes->addAction(addBorderAction(i18n("Very Large"), "border_verylarge_action", KDecorationDefines::BorderVeryLarge));
    borderSizes->addAction(addBorderAction(i18n("Huge"), "border_huge_action", KDecorationDefines::BorderHuge));
    borderSizes->addAction(addBorderAction(i18n("Very Huge"), "border_veryhuge_action", KDecorationDefines::BorderVeryHuge));
    borderSizes->addAction(addBorderAction(i18n("Oversized"), "border_oversized_action", KDecorationDefines::BorderOversized));
    actionCollection()->action("border_normal_action")->setChecked(true);
    connect(borderSizes, SIGNAL(triggered(QAction*)), SLOT(borderSizesTriggered(QAction*)));

    QActionGroup *buttonSizes = new QActionGroup(this);
    buttonSizes->addAction(addBorderAction(i18n("Tiny"), "button_tiny_action", KDecorationDefines::BorderTiny));
    buttonSizes->addAction(addBorderAction(i18n("Normal"), "button_normal_action", KDecorationDefines::BorderNormal));
    buttonSizes->addAction(addBorderAction(i18n("Large"), "button_large_action", KDecorationDefines::BorderLarge));
    buttonSizes->addAction(addBorderAction(i18n("Very Large"), "button_verylarge_action", KDecorationDefines::BorderVeryLarge));
    buttonSizes->addAction(addBorderAction(i18n("Huge"), "button_huge_action", KDecorationDefines::BorderHuge));
    buttonSizes->addAction(addBorderAction(i18n("Very Huge"), "button_veryhuge_action", KDecorationDefines::BorderVeryHuge));
    buttonSizes->addAction(addBorderAction(i18n("Oversized"), "button_oversized_action", KDecorationDefines::BorderOversized));
    actionCollection()->action("button_normal_action")->setChecked(true);
    connect(buttonSizes, SIGNAL(triggered(QAction*)), SLOT(buttonSizesTriggered(QAction*)));

    for (int i=Aurorae::MinimizeButton; i<=Aurorae::MenuButton; ++i) {
        if (i==Aurorae::RestoreButton) {
            continue;
        }
        setupButtonAction(i, true);
        setupButtonAction(i, false);
    }
}

KAction* AuroraeDesigner::addBorderAction(const QString& title, const QString& name, int border)
{
    KAction *a = new KAction(title, this);
    a->setData(border);
    a->setCheckable(true);
    actionCollection()->addAction(name, a);
    return a;
}

void AuroraeDesigner::setupButtonAction(int type, bool left)
{
    KAction *a = new KAction(this);
    a->setData(type);
    a->setCheckable(true);
    a->setEnabled(false);
    switch (type) {
    case Aurorae::MinimizeButton:
        a->setText(i18n("Minimize"));
        break;
    case Aurorae::MaximizeButton: // fall through
    case Aurorae::RestoreButton:
        a->setText(i18n("Maximize"));
        break;
    case Aurorae::CloseButton:
        a->setText(i18n("Close"));
        break;
    case Aurorae::AllDesktopsButton:
        a->setText(i18n("All Desktops"));
        break;
    case Aurorae::KeepAboveButton:
        a->setText(i18n("Keep Above"));
        break;
    case Aurorae::KeepBelowButton:
        a->setText(i18n("Keep Below"));
        break;
    case Aurorae::ShadeButton:
        a->setText(i18n("Shade"));
        break;
    case Aurorae::HelpButton:
        a->setText(i18n("Help"));
        break;
    case Aurorae::MenuButton:
        a->setText(i18n("Menu"));
        break;
    default:
        break; // nothing
    }
    QString actionName = (left ? QString("left") : QString("right")) + QString("_button_");
    actionCollection()->addAction(actionName + Aurorae::AuroraeTheme::mapButtonToName((Aurorae::AuroraeButtonType)type), a);
    if (left) {
        connect(a, SIGNAL(triggered(bool)), m_view, SLOT(leftButtonTriggered(bool)));
    } else {
        connect(a, SIGNAL(triggered(bool)), m_view, SLOT(rightButtonTriggered(bool)));
    }
}

void AuroraeDesigner::enableButtonActions()
{
    for (int i=Aurorae::MinimizeButton; i<=Aurorae::MenuButton; ++i) {
        if (i == Aurorae::RestoreButton) {
            continue;
        }
        char c = Aurorae::AuroraeTheme::mapButtonToChar((Aurorae::AuroraeButtonType)i);
        const QString buttonName = QString("_button_") + Aurorae::AuroraeTheme::mapButtonToName((Aurorae::AuroraeButtonType)i);
        bool enable = m_theme->hasButton((Aurorae::AuroraeButtonType)i);
        if (i == Aurorae::MenuButton) {
            enable = true;
        }
        QAction *left = actionCollection()->action(QString("left") + buttonName);
        left->setEnabled(enable);
        left->setChecked(m_theme->defaultButtonsLeft().contains(c) && enable);
        QAction *right = actionCollection()->action(QString("right") + buttonName);
        right->setEnabled(enable);
        right->setChecked(m_theme->defaultButtonsRight().contains(c) && enable);
    }
}

void AuroraeDesigner::fileNew()
{
    // this slot is called whenever the File->New menu is selected,
    // the New shortcut is pressed (usually CTRL+N) or the New toolbar
    // button is clicked

    // create a new window
    (new AuroraeDesigner)->show();
}

void AuroraeDesigner::fileOpen()
{
    QWeakPointer<KDialog> dlg = new KDialog(this);
    dlg.data()->setButtons(KDialog::Ok|KDialog::Cancel);
    dlg.data()->setWindowTitle(i18n("Open Theme"));
    QListView *view = new QListView(dlg.data());
    view->setModel(new PreviewModel(view));
    view->setItemDelegate(new PreviewDelegate(view));
    dlg.data()->setMainWidget(view);
    dlg.data()->resize(650, 500);
    if (dlg.data()->exec() == KDialog::Accepted) {
        // do something
        const QModelIndex &index = view->currentIndex();
        loadTheme(view->model()->data(index, Qt::ToolTipRole).toString());
    }
    delete dlg.data();
}

void AuroraeDesigner::configChanged()
{
    m_theme->readThemeConfig(*m_config.data());
    enableButtonActions();
}

void AuroraeDesigner::optionsPreferences()
{
    if ( KConfigDialog::showDialog( "settings" ) ) {
        return;
    }
    KConfigDialog *dialog = new KConfigDialog(this, "settings", Settings::self());
    QWidget *generalSettingsDlg = new QWidget;
    ui_prefs_base.setupUi(generalSettingsDlg);
    dialog->addPage(generalSettingsDlg, i18n("General"), "package_setting");
    connect(dialog, SIGNAL(settingsChanged(QString)), m_view, SLOT(settingsChanged()));
    dialog->setAttribute( Qt::WA_DeleteOnClose );
    dialog->show();
}

void AuroraeDesigner::save()
{
    m_config->reparseConfiguration();
    const QString &theme = m_theme->themeName();
    KConfig conf("aurorae/themes/" + theme + '/' + theme + "rc", KConfig::SimpleConfig, "data");
    // General
    KConfigGroup general(&conf, "General");
    general.deleteGroup();
    QMap<QString, QString> entries = m_config->entryMap("General");
    QMap<QString, QString>::const_iterator it = entries.constBegin();
    for(; it != entries.constEnd(); ++it) {
        general.writeEntry(it.key(), it.value());
    }
    // Layout
    KConfigGroup layout(&conf, "Layout");
    layout.deleteGroup();
    entries = m_config->entryMap("Layout");
    it = entries.constBegin();
    for(; it != entries.constEnd(); ++it) {
        layout.writeEntry(it.key(), it.value());
    }
    conf.sync();
    m_desktopFile->reparseConfiguration();
    KConfig df("aurorae/themes/" + theme + "/metadata.desktop", KConfig::SimpleConfig, "data");
    KConfigGroup desktop(&df, "Desktop Entry");
    desktop.deleteGroup();
    entries = m_desktopFile->entryMap("Desktop Entry");
    it = entries.constBegin();
    for(; it != entries.constEnd(); ++it) {
        desktop.writeEntry(it.key(), it.value());
    }
    desktop.sync();
}

void AuroraeDesigner::reload()
{
    loadTheme(m_theme->themeName());
    enableButtonActions();
}

void AuroraeDesigner::loadTheme(const QString &theme)
{
    KConfig conf("aurorae/themes/" + theme + '/' + theme + "rc", KConfig::SimpleConfig, "data");
    // Copy config to temporary file
    QMap<QString, QString> entries = conf.entryMap("General");
    QMap<QString, QString>::const_iterator it = entries.constBegin();
    KConfigGroup tmpgroup(m_config, "General");
    tmpgroup.deleteGroup();
    for(; it != entries.constEnd(); ++it) {
        tmpgroup.writeEntry(it.key(), it.value());
    }
    tmpgroup.sync();
    entries = conf.entryMap("Layout");
    it = entries.constBegin();
    KConfigGroup tmpgroup2(m_config, "Layout");
    tmpgroup2.deleteGroup();
    for(; it != entries.constEnd(); ++it)
        tmpgroup2.writeEntry(it.key(), it.value());
    tmpgroup2.sync();
    KConfig df("aurorae/themes/" + theme + "/metadata.desktop", KConfig::SimpleConfig, "data");
    entries = df.entryMap("Desktop Entry");
    it = entries.constBegin();
    KConfigGroup desktopGroup(m_desktopFile, "Desktop Entry");
    desktopGroup.deleteGroup();
    for(; it != entries.constEnd(); ++it)
        desktopGroup.writeEntry(it.key(), it.value());
    desktopGroup.sync();
    m_theme->loadTheme(theme, *m_config.data());
    m_model->setConfigGroups(m_config, m_desktopFile);
    actionCollection()->action("toggle_active_action")->setEnabled(true);
    actionCollection()->action("toggle_maximize_action")->setEnabled(true);
    actionCollection()->action("reload_action")->setEnabled(true);
    actionCollection()->action("toggle_compositing_action")->setEnabled(true);
    actionCollection()->action("compliance_action")->setEnabled(true);
    actionCollection()->action("show_tooltips_action")->setEnabled(true);
    actionCollection()->action("add_tab_action")->setEnabled(true);
    enableButtonActions();
}

void AuroraeDesigner::compositingChanged(bool enabled)
{
    m_theme->setCompositingActive(enabled);
    m_view->updateScene(QList<QRectF>() << m_view->sceneRect());
}

void AuroraeDesigner::complianceDialog()
{
    KDialog *dlg = new KDialog(this);
    QTreeView *view = new QTreeView(dlg);
    view->setModel(new ComplianceModel(m_theme, view));
    view->setItemDelegate(new ComplianceDelegate(view));
    dlg->setWindowTitle(i18n("Theme Compliance"));
    dlg->setMainWidget(view);
    dlg->setButtons(KDialog::Close);
    dlg->show();
}

void AuroraeDesigner::borderSizesTriggered(QAction *action)
{
    m_theme->setBorderSize((KDecorationDefines::BorderSize)action->data().toInt());
    m_view->updateScene(QList<QRectF>() << m_view->sceneRect());
}

void AuroraeDesigner::buttonSizesTriggered(QAction *action)
{
    m_theme->setButtonSize((KDecorationDefines::BorderSize)action->data().toInt());
    m_view->updateScene(QList<QRectF>() << m_view->sceneRect());
}

void AuroraeDesigner::ghns()
{
    QPointer<KNS3::DownloadDialog> downloadDialog = new KNS3::DownloadDialog("aurorae.knsrc", this);
    downloadDialog->exec();
    delete downloadDialog;
}

void AuroraeDesigner::enableRemoveTab(bool enable)
{
    actionCollection()->action("remove_tab_action")->setEnabled(enable);
}


#include "auroraedesigner.moc"
