/*
    Aurorae Designer: an application to design Aurorae themes.
    Copyright (C) 2010  Martin Gräßlin <kde@martin-graesslin.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/
#include "auroraedesignerview.h"
#include <settings.h>
// Aurorae
#include <auroraescene.h>
#include <auroraetheme.h>
// Qt
#include <QtCore/QTimer>
#include <QtGui/QAction>
#include <QtGui/qevent.h>
#include <QtGui/QGraphicsLinearLayout>
#include <QtGui/QGraphicsWidget>
// KDE
#include <KDE/KLocale>
#include <KDE/KIcon>

using namespace Aurorae;

AuroraeDesignerView::AuroraeDesignerView(QWidget* parent, AuroraeTheme* theme)
    : QGraphicsView(parent)
    , m_theme(theme)
    , m_showGrid(false)
{
    setAttribute(Qt::WA_TranslucentBackground);
    setFrameShape(QFrame::NoFrame);
    QPalette pal = palette();
    pal.setColor(backgroundRole(), Settings::self()->col_background());
    setPalette(pal);
    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

    AuroraeScene *scene = new AuroraeScene(m_theme, QString(), QString(), true, this);
    setScene(scene);
    scene->setIcon(KIcon("xorg"));
    scene->setActive(true, false);
    connect(scene, SIGNAL(maximize(Qt::MouseButtons)), SLOT(slotMaximizeChanged(Qt::MouseButtons)));
    connect(this, SIGNAL(maximized(bool)), SLOT(slotMaximizeChanged(bool)));
    connect(scene, SIGNAL(toggleKeepAbove()), SLOT(slotKeepAboveToggled()));
    connect(scene, SIGNAL(toggleKeepBelow()), SLOT(slotKeepBelowToggled()));
    connect(scene, SIGNAL(toggleOnAllDesktops()), SLOT(slotAllDesktopsToggled()));
    connect(scene, SIGNAL(toggleShade()), SLOT(slotShadeToggled()));
    connect(scene, SIGNAL(tabMouseButtonRelease(QGraphicsSceneMouseEvent*,int)), SLOT(tabClicked(QGraphicsSceneMouseEvent*,int)));
    connect(m_theme, SIGNAL(themeChanged()), SLOT(themeChanged()));
}

AuroraeDesignerView::~AuroraeDesignerView()
{
}

void AuroraeDesignerView::resizeEvent(QResizeEvent* event)
{
    QGraphicsView::resizeEvent(event);
    scene()->setSceneRect(QRectF(QPointF(0, 0), event->size()));

    static_cast<AuroraeScene*>(scene())->updateLayout();
}

void AuroraeDesignerView::drawForeground(QPainter *painter, const QRectF &rect)
{
    QGraphicsView::drawForeground(painter, rect);
    if (m_showGrid) {
        painter->setClipRect(rect);
        drawGrid(painter);
    }
}

void AuroraeDesignerView::drawGrid(QPainter *painter)
{
    painter->save();
    painter->setPen(Settings::self()->col_gridcolor());
    bool maximized = static_cast<AuroraeScene*>(scene())->maximizeMode() == KDecorationDefines::MaximizeFull;
    int paddingLeft, paddingTop, paddingRight, paddingBottom;
    m_theme->padding(paddingLeft, paddingTop, paddingRight, paddingBottom);
    // left padding
    painter->drawLine(paddingLeft, paddingTop,
                      paddingLeft, height() - paddingBottom);
    // right padding
    painter->drawLine(width() - paddingRight, paddingTop,
                      width() - paddingRight, height() - paddingBottom);
    // top padding
    painter->drawLine(paddingLeft, paddingTop,
                      width() - paddingRight, paddingTop);
    // bottom padding
    painter->drawLine(paddingLeft, height() - paddingBottom,
                      width() - paddingRight, height() - paddingBottom);

    // border left
    int borderLeft, borderTop, borderRight, borderBottom;
    m_theme->borders(borderLeft, borderTop, borderRight, borderBottom, maximized);
    const qreal top = paddingTop + borderTop;
    painter->drawLine(paddingLeft + borderLeft, top,
                      paddingLeft + borderLeft,
                      height() - paddingBottom - borderBottom);
    // border right
    painter->drawLine(width() - paddingRight - borderRight, top,
                      width() - paddingRight - borderRight,
                      height() - paddingBottom - borderBottom);
    // border top
    painter->drawLine(paddingLeft + borderLeft, top,
                      width() - paddingRight - borderRight, top);
    // border bottom
    painter->drawLine(paddingLeft + borderLeft,
                      height() - paddingBottom - borderBottom,
                      width() - paddingRight - borderRight,
                      height() - paddingBottom - borderBottom);
    // title edge top
    int titleEdgeLeft, titleEdgeTop, titleEdgeRight, titleEdgeBottom;
    m_theme->titleEdges(titleEdgeLeft, titleEdgeTop, titleEdgeRight, titleEdgeBottom, maximized);
    painter->drawLine(paddingLeft, paddingTop + titleEdgeTop,
                      width() - paddingRight, paddingTop + titleEdgeTop);
    // title edge bottom
    painter->drawLine(paddingLeft, top - titleEdgeBottom,
                      width() - paddingRight, top - titleEdgeBottom);
    // title edge left
    painter->drawLine(paddingLeft + titleEdgeLeft,
                      paddingTop + titleEdgeTop,
                      paddingLeft + titleEdgeLeft, top - titleEdgeBottom);
    // title edge right
    painter->drawLine(width() - paddingRight - titleEdgeRight,
                      paddingTop + titleEdgeTop,
                      width() - paddingRight - titleEdgeRight,
                      top - titleEdgeBottom);
    // button margin top left
    int buttonMarginTop, titleBorderLeft, titleBorderRight;
    m_theme->buttonMargins(titleBorderLeft, buttonMarginTop, titleBorderRight);
    const int leftButtonsWidth  = static_cast<AuroraeScene*>(scene())->leftButtonsWidth();
    const int rightButtonsWidth = static_cast<AuroraeScene*>(scene())->rightButtonsWidth();
    painter->drawLine(paddingLeft + titleEdgeLeft,
                      paddingTop + titleEdgeTop + buttonMarginTop,
                      paddingLeft + titleEdgeLeft + leftButtonsWidth,
                      paddingTop + titleEdgeTop + buttonMarginTop);
    // button margin top right
    painter->drawLine(width() - paddingRight - titleEdgeRight - rightButtonsWidth,
                      paddingTop + titleEdgeTop + buttonMarginTop,
                      width() - paddingRight - titleEdgeRight,
                      paddingTop + titleEdgeTop + buttonMarginTop);
    // title border left
    painter->drawLine(paddingLeft + titleEdgeLeft + leftButtonsWidth,
                      paddingTop + titleEdgeTop + buttonMarginTop,
                      paddingLeft + titleEdgeLeft + leftButtonsWidth,
                      top - titleEdgeBottom);
    // title border right
    painter->drawLine(width() - paddingRight - titleEdgeRight - rightButtonsWidth,
                      paddingTop + titleEdgeTop + buttonMarginTop,
                      width() - paddingRight - titleEdgeRight - rightButtonsWidth,
                      top - titleEdgeBottom);
    // title left
    painter->drawLine(paddingLeft + titleEdgeLeft + titleBorderLeft + leftButtonsWidth,
                      paddingTop + titleEdgeTop,
                      paddingLeft + titleEdgeLeft + titleBorderLeft + leftButtonsWidth,
                      top - titleEdgeBottom);
    // title right
    painter->drawLine(width() - paddingRight - titleEdgeRight - titleBorderRight - rightButtonsWidth,
                      paddingTop + titleEdgeTop,
                      width() - paddingRight - titleEdgeRight - titleBorderRight - rightButtonsWidth,
                      top - titleEdgeBottom);
    painter->restore();
}

void AuroraeDesignerView::slotShowGrid(bool enabled)
{
    m_showGrid = enabled;
    updateScene( QList<QRectF>() << sceneRect());
}

void AuroraeDesignerView::slotActiveChanged(bool active)
{
    static_cast<AuroraeScene*>(scene())->setActive(active);
}

void AuroraeDesignerView::slotMaximizeChanged(bool maximized)
{
    KDecorationDefines::MaximizeMode mode =
        maximized ? KDecorationDefines::MaximizeFull : KDecorationDefines::MaximizeRestore;
    static_cast<AuroraeScene*>(scene())->setMaximizeMode(mode);
}

void AuroraeDesignerView::slotMaximizeChanged(Qt::MouseButtons buttons)
{
    if (buttons.testFlag(Qt::LeftButton)) {
        AuroraeScene *s = static_cast<AuroraeScene*>(scene());
        bool enabled = s->maximizeMode() != KDecorationDefines::MaximizeFull;
        emit maximized(enabled);
    }
}

void AuroraeDesignerView::slotAllDesktopsToggled()
{
    AuroraeScene *s = static_cast<AuroraeScene*>(scene());
    s->setAllDesktops(!s->isAllDesktops());
}

void AuroraeDesignerView::slotKeepAboveToggled()
{
    AuroraeScene *s = static_cast<AuroraeScene*>(scene());
    s->setKeepAbove(!s->isKeepAbove());
    if (s->isKeepAbove()) {
        s->setKeepBelow(false);
    }
}

void AuroraeDesignerView::slotKeepBelowToggled()
{
    AuroraeScene *s = static_cast<AuroraeScene*>(scene());
    s->setKeepBelow(!s->isKeepBelow());
    if (s->isKeepBelow()) {
        s->setKeepAbove(false);
    }
}

void AuroraeDesignerView::slotShadeToggled()
{
    AuroraeScene *s = static_cast<AuroraeScene*>(scene());
    s->setShade(!s->isShade());
}

void AuroraeDesignerView::settingsChanged()
{
    QPalette pal = palette();
    pal.setColor(backgroundRole(), Settings::self()->col_background());
    setPalette(pal);
}

void AuroraeDesignerView::themeChanged()
{
    QTimer::singleShot(0, this, SLOT(themeChangedTimeout()));
}

void AuroraeDesignerView::themeChangedTimeout()
{
    AuroraeScene *s = static_cast<AuroraeScene*>(scene());
    s->setButtons(m_theme->defaultButtonsLeft(), m_theme->defaultButtonsRight());
    if (s->tabCount() == 0) {
        s->addTab(m_theme->themeName());
    } else {
        s->setCaption(m_theme->themeName());
    }
}

void AuroraeDesignerView::leftButtonTriggered(bool active)
{
    QAction *a = dynamic_cast<QAction*>(QObject::sender());
    if (!a) {
        return;
    }
    AuroraeScene *s = static_cast<AuroraeScene*>(scene());
    QString buttons = s->leftButtons();
    const char c = AuroraeTheme::mapButtonToChar((AuroraeButtonType)a->data().toInt());
    if (active) {
        buttons.append(c);
    } else {
        buttons.remove(c);
    }
    s->setButtons(buttons, s->rightButtons());
}

void AuroraeDesignerView::rightButtonTriggered(bool active)
{
    QAction *a = dynamic_cast<QAction*>(QObject::sender());
    if (!a) {
        return;
    }
    AuroraeScene *s = static_cast<AuroraeScene*>(scene());
    QString buttons = s->rightButtons();
    const char c = AuroraeTheme::mapButtonToChar((AuroraeButtonType)a->data().toInt());
    if (active) {
        buttons.append(c);
    } else {
        buttons.remove(c);
    }
    s->setButtons(s->leftButtons(), buttons);
}

void AuroraeDesignerView::addTab()
{
    AuroraeScene *s = static_cast<AuroraeScene*>(scene());
    s->addTab(i18n("Tab %1", s->tabCount()));
    s->setFocusedTab(s->tabCount()-1);
    emit enableRemoveTab(true);
}

void AuroraeDesignerView::removeTab()
{
    AuroraeScene *s = static_cast<AuroraeScene*>(scene());
    if (s->tabCount() > 1) {
        const int tab = s->focusedTab();
        s->removeLastTab();
        if (s->tabCount() <= tab) {
            s->setFocusedTab(s->tabCount()-1);
        }
        if (s->tabCount() == 1) {
            emit enableRemoveTab(false);
        }
    }
}

void AuroraeDesignerView::tabClicked(QGraphicsSceneMouseEvent* event, int index)
{
    Q_UNUSED(event)
    static_cast<AuroraeScene*>(scene())->setFocusedTab(index);
}

#include "auroraedesignerview.moc"
