/*
    Aurorae Designer: an application to design Aurorae themes.
    Copyright (C) 2010  Martin Gräßlin <kde@martin-graesslin.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/

#include "compliancemodel.h"
#include <auroraetheme.h>
#include <KDE/KLocale>
#include <KDE/Plasma/FrameSvg>

ComplianceModel::ComplianceModel(Aurorae::AuroraeTheme* theme, QObject* parent)
    : QAbstractItemModel(parent)
    , m_theme(theme)
{
}

QVariant ComplianceModel::data(const QModelIndex &index, int role) const
{
    if (role != Qt::DisplayRole || !index.isValid()) {
        return QVariant();
    }
    if (!index.parent().isValid() && index.row() < TopLevelCount) {
        if (index.column() == 0) {
            return mapIdToTopLevelDescription(index.row());
        } else {
            if (index.row() == Decoration) {
                return m_theme->decoration() != NULL && m_theme->decoration()->hasElementPrefix("decoration");
            } else {
                return m_theme->hasButton((Aurorae::AuroraeButtonType)mapIdToAuroraeButton(index.row()));
            }
        }
        return QVariant();
    }
    if (index.parent().isValid() && index.parent().internalId() < TopLevelCount) {
        if (index.parent().internalId() == Decoration) {
            if (index.row() < DecorationCount) {
                if (index.column() == 0) {
                    return mapIdToDecorationDescription(index.row());
                } else {
                    return m_theme->decoration()->hasElementPrefix(mapIdToDecorationElement(index.row()));
                }
            }
        } else {
            if (index.row() < ButtonCount) {
                if (index.column() == 0) {
                    return mapIdToButtonDescription(index.row());
                } else {
                    return m_theme->button((Aurorae::AuroraeButtonType)mapIdToAuroraeButton(index.parent().internalId()))->hasElementPrefix(mapIdToButtonElement(index.row()));
                }
            }
        }
    }
    return QVariant();
}

int ComplianceModel::columnCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent)
    return 2;
}

int ComplianceModel::rowCount(const QModelIndex &parent) const
{
    if (!parent.isValid()) {
        return TopLevelCount;
    } else {
        if (parent.internalId() == Decoration) {
            return DecorationCount;
        } else if (parent.internalId() < TopLevelCount) {
            if (m_theme->hasButton((Aurorae::AuroraeButtonType)mapIdToAuroraeButton(parent.internalId()))) {
                return ButtonCount;
            } else {
                return 0;
            }
        }
    }
    return 0;
}

QModelIndex ComplianceModel::parent(const QModelIndex &child) const
{
    if (child.internalId() < TopLevelCount) {
        return QModelIndex();
    }
    const int id = (child.internalId()-100-child.row())/100;
    return createIndex(id, 0, id);
}

QModelIndex ComplianceModel::index(int row, int column, const QModelIndex &parent) const
{
    if (column > 1) {
        return QModelIndex();
    }
    if (!parent.isValid() && row < TopLevelCount) {
        return createIndex(row, column, row);
    }
    if (parent.isValid()) {
        if (parent.internalId() == Decoration) {
            if (row < DecorationCount) {
                return createIndex(row, column, 100+row);
            }
        } else if (parent.internalId() < TopLevelCount) {
            return createButtonIndex(row, column, parent);
        }
    }
    return QModelIndex();
}

QModelIndex ComplianceModel::createButtonIndex(int row, int column, const QModelIndex &parent) const
{
    if (m_theme->hasButton((Aurorae::AuroraeButtonType)mapIdToAuroraeButton(parent.internalId()))) {
        if (row < ButtonCount) {
            return createIndex(row, column, 100*(int)parent.internalId()+100+row);
        }
    }
    return QModelIndex();
}

QVariant ComplianceModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role == Qt::DisplayRole && orientation == Qt::Horizontal) {
        switch (section) {
        case 0:
            return i18n("Feature");
        case 1:
            return i18n("Supported");
        default:
            break; // fallback to default
        }
    }
    return QAbstractItemModel::headerData(section, orientation, role);
}

QString ComplianceModel::mapIdToDecorationElement(int id) const
{
    switch (id) {
    case InactiveElement:
        return "decoration-inactive";
    case OpaqueElement:
        return "decoration-opaque";
    case OpaueInactiveElement:
        return "decoration-opaque-inactive";
    case MaximizedElement:
        return "decoration-maximized";
    case MaximizedInactiveElement:
        return "decoration-maximized-inactive";
    case MaximizedOpaqueElement:
        return "decoration-maximized-opaque";
    case MaximizedOpaqueInactiveElement:
        return "decoration-maximized-opaque-inactive";
    case InnerBorderElement:
        return "innerborder";
    case InnerBorderInactiveElement:
        return "innerborder-inactive";
    case TabActiveFocused:
        return "tab-active-focused";
    case TabActiveUnfocused:
        return "tab-active-unfocused";
    case TabInactiveFocused:
        return "tab-inactive-focused";
    case TabInactiveUnfocused:
        return "tab-inactive-unfocused";
    default:
        return QString();
    }
}

QString ComplianceModel::mapIdToDecorationDescription(int id) const
{
    switch (id) {
    case InactiveElement:
        return i18n("Inactive");
    case OpaqueElement:
        return i18n("Opaque");
    case OpaueInactiveElement:
        return i18n("Inactive Opaque");
    case MaximizedElement:
        return i18n("Maximized");
    case MaximizedInactiveElement:
        return i18n("Maximized Inactive");
    case MaximizedOpaqueElement:
        return i18n("Maximized Opaque");
    case MaximizedOpaqueInactiveElement:
        return i18n("Maximized Inactive Opaque");
    case InnerBorderElement:
        return i18n("Inner Border");
    case InnerBorderInactiveElement:
        return i18n("Inner Border Inactive");
    case TabActiveFocused:
        return i18n("Focused Tab of Active Window");
    case TabActiveUnfocused:
        return i18n("Unfocused Tab of Active Window");
    case TabInactiveFocused:
        return i18n("Focused Tab of Inactive Window");
    case TabInactiveUnfocused:
        return i18n("Unfocused Tab of Inactive Window");
    default:
        return QString();
    }
}

int ComplianceModel::mapIdToAuroraeButton(qint64 id) const
{
    switch (id) {
    case MinimizeButton:
        return Aurorae::MinimizeButton;
    case MaximizeButton:
        return Aurorae::MaximizeButton;
    case RestoreButton:
        return Aurorae::RestoreButton;
    case CloseButton:
        return Aurorae::CloseButton;
    case AllDesktopsButton:
        return Aurorae::AllDesktopsButton;
    case KeepAboveButton:
        return Aurorae::KeepAboveButton;
    case KeepBelowButton:
        return Aurorae::KeepBelowButton;
    case ShadeButton:
        return Aurorae::ShadeButton;
    case HelpButton:
        return Aurorae::HelpButton;
    case MenuButton:
        return Aurorae::MenuButton;
    default:
        return -1;
    }
}

QString ComplianceModel::mapIdToButtonElement(int id) const
{
    switch (id) {
    case InactiveButtonElement:
        return "inactive";
    case HoverElement:
        return "hover";
    case HoverInactiveElement:
        return "hover-inactive";
    case PressedElement:
        return "pressed";
    case PressedInactiveElement:
        return "pressed-inactive";
    case DeactivatedElement:
        return "deactivated";
    case DeactivatedInactiveElement:
        return "deactivated-inactive";
    default:
        return QString();
    }
}

QString ComplianceModel::mapIdToButtonDescription(int id) const
{
    switch (id) {
    case InactiveButtonElement:
        return i18n("Inactive");
    case HoverElement:
        return i18n("Hover");
    case HoverInactiveElement:
        return i18n("Hover Inactive");
    case PressedElement:
        return i18n("Pressed");
    case PressedInactiveElement:
        return i18n("Pressed Inactive");
    case DeactivatedElement:
        return i18n("Deactivated");
    case DeactivatedInactiveElement:
        return i18n("Deactivate Inactive");
    default:
        return QString();
    }
}

QString ComplianceModel::mapIdToTopLevelDescription(int id) const
{
    switch (id) {
    case Decoration:
        return i18n("Decoration");
    case MinimizeButton:
        return i18n("Minimize Button");
    case MaximizeButton:
        return i18n("Maximize Button");
    case RestoreButton:
        return i18n("Restore Button");
    case CloseButton:
        return i18n("Close Button");
    case AllDesktopsButton:
        return i18n("All Desktops Button");
    case KeepAboveButton:
        return i18n("Keep Above Button");
    case KeepBelowButton:
        return i18n("Keep Below Button");
    case ShadeButton:
        return i18n("Shade Button");
    case HelpButton:
        return i18n("Help Button");
    case MenuButton:
        return i18n("Menu Button");
    default:
        return QString();
    }
}
