/*
    Aurorae Designer: an application to design Aurorae themes.
    Copyright (C) 2010  Martin Gräßlin <kde@martin-graesslin.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/

#include "previewmodel.h"
// Aurorae
#include <auroraescene.h>
#include <auroraetheme.h>
// Qt
#include <QtCore/QTimer>
#include <QtGui/QApplication>
#include <QtGui/QStyle>
#include <QtGui/QPainter>
// KDE
#include <KDE/KConfigGroup>
#include <KDE/KDesktopFile>
#include <KDE/KIcon>
#include <KDE/KGlobal>
#include <KDE/KLocale>
#include <KDE/KStandardDirs>
#include <KDebug>

PreviewModel::PreviewModel(QObject* parent)
    : QAbstractItemModel(parent)
    , m_theme(new Aurorae::AuroraeTheme(this))
{
    // delay for snappy interface
    QTimer::singleShot(100, this, SLOT(init()));
}

void PreviewModel::init()
{
    beginResetModel();
    Aurorae::AuroraeScene *scene = new Aurorae::AuroraeScene(m_theme, QString(), QString(), true, this);
    scene->setIcon(KIcon("xorg"));
    scene->setActive(true, false);
    scene->setSceneRect(0, 0, 600, 200);
    // get all themes
    QStringList files = KGlobal::dirs()->findAllResources("data",
                                                            "aurorae/themes/*/metadata.desktop",
                                                            KStandardDirs::NoDuplicates);
    foreach(const QString &theme, files) {
        const int themeSepIndex = theme.lastIndexOf( '/', -1 );
        const QString themeRoot = theme.left( themeSepIndex );
        const int themeNameSepIndex = themeRoot.lastIndexOf( '/', -1 );
        const QString packageName = themeRoot.right( themeRoot.length() - themeNameSepIndex - 1 );

        KDesktopFile df( theme );
        const QString name = df.desktopGroup().readEntry("X-KDE-PluginInfo-Name", packageName);
        const QString themeName = df.readName();
        KConfig config("aurorae/themes/" + name + '/' + name + "rc", KConfig::SimpleConfig, "data");
        m_theme->loadTheme(name, config);
        if (!m_theme->isValid()) {
            kDebug() << "Theme " << name << " is not valid - skipping";
            continue;
        }
        m_themeNames << name;
        scene->setButtons(m_theme->defaultButtonsLeft(), m_theme->defaultButtonsRight());
        QImage image(QSize(600, 200), QImage::Format_ARGB32);
        image.fill(Qt::transparent);
        QPainter painter(&image);
        int left, top, right, bottom;
        m_theme->borders(left, top, right, bottom, false);
        int padLeft, padRight, padTop, padBottom;
        m_theme->padding(padLeft, padTop, padRight, padBottom);
        top = qMin(int( top * .9 ), 30);
        int xoffset = qMin(qMax(10, QApplication::isRightToLeft() ? left : right), 30);
        scene->setSceneRect(0, 0 ,
                            image.width() - xoffset - 20 + padLeft + padRight,
                            image.height() - top - 20 + padLeft + padRight );
        scene->setActive(false, false);
        scene->addTab(themeName + " - " + i18n("Inactive Window"));
        scene->updateLayout();
        QRect rect = QRectF(QPointF(10 + xoffset - padLeft, 10 - padTop), scene->sceneRect().size()).toRect();
        scene->render(&painter, QStyle::visualRect( QApplication::layoutDirection(), image.rect(), rect));
        scene->setActive(true, false);
        scene->setCaption(themeName + " - " + i18n("Active Window"));
        rect = QRectF(QPointF(10 - padLeft, top + 10 - padTop), scene->sceneRect().size()).toRect();
        scene->render(&painter, QStyle::visualRect(QApplication::layoutDirection(), image.rect(), rect));
        painter.end();
        m_images << image;
    }
    endResetModel();
}

QVariant PreviewModel::data(const QModelIndex& index, int role) const
{
    if (!index.isValid() || index.row() > m_themeNames.count()) {
        return QVariant();
    }
    if (role == Qt::DisplayRole) {
        return m_images[index.row()];
    } else if (role == Qt::ToolTipRole) {
        return m_themeNames[index.row()];
    } else {
        return QVariant();
    }
}

int PreviewModel::columnCount(const QModelIndex& parent) const
{
    Q_UNUSED(parent)
    return 1;
}

int PreviewModel::rowCount(const QModelIndex& parent) const
{
    Q_UNUSED(parent)
    return m_themeNames.count();
}

QModelIndex PreviewModel::parent(const QModelIndex& child) const
{
    Q_UNUSED(child)
    return QModelIndex();
}

QModelIndex PreviewModel::index(int row, int column, const QModelIndex& parent) const
{
    Q_UNUSED(parent)
    if (row < m_themeNames.count()) {
        return createIndex(row, column);
    }
    return QModelIndex();
}
